@@ -1,16 +1,17 @@
 from zope.interface import implements
 from twisted.trial import unittest
 from twisted.internet import defer, reactor
-from twisted.internet.interfaces import IConsumer
 from twisted.python.failure import Failure
-from foolscap import eventual
+from foolscap.api import fireEventually
 from allmydata import hashtree, uri
 from allmydata.immutable import encode, upload, download
 from allmydata.util import hashutil
 from allmydata.util.assertutil import _assert
-from allmydata.interfaces import IStorageBucketWriter, IStorageBucketReader, NotEnoughSharesError
+from allmydata.util.consumer import MemoryConsumer
+from allmydata.interfaces import IStorageBucketWriter, IStorageBucketReader, \
+     NotEnoughSharesError, IStorageBroker, UploadUnhappinessError
 from allmydata.monitor import Monitor
-import common_util as testutil
+import allmydata.test.common_util as testutil
 
 class LostPeerError(Exception):
     pass
@@ -18,14 +19,13 @@ class LostPeerError(Exception):
 def flip_bit(good): # flips the last bit
     return good[:-1] + chr(ord(good[-1]) ^ 0x01)
 
-class FakeClient:
-    def log(self, *args, **kwargs):
-        pass
+class FakeStorageBroker:
+    implements(IStorageBroker)
 
 class FakeBucketReaderWriterProxy:
     implements(IStorageBucketWriter, IStorageBucketReader)
     # these are used for both reading and writing
-    def __init__(self, mode="good"):
+    def __init__(self, mode="good", peerid="peer"):
         self.mode = mode
         self.blocks = {}
         self.plaintext_hashes = []
@@ -33,14 +33,15 @@ class FakeBucketReaderWriterProxy:
         self.block_hashes = None
         self.share_hashes = None
         self.closed = False
+        self.peerid = peerid
 
     def get_peerid(self):
-        return "peerid"
+        return self.peerid
 
     def _start(self):
         if self.mode == "lost-early":
             f = Failure(LostPeerError("I went away early"))
-            return eventual.fireEventually(f)
+            return fireEventually(f)
         return defer.succeed(self)
 
     def put_header(self):
@@ -49,7 +50,7 @@ class FakeBucketReaderWriterProxy:
     def put_block(self, segmentnum, data):
         if self.mode == "lost-early":
             f = Failure(LostPeerError("I went away early"))
-            return eventual.fireEventually(f)
+            return fireEventually(f)
         def _try():
             assert not self.closed
             assert segmentnum not in self.blocks
@@ -174,6 +175,7 @@ def make_data(length):
     return data[:length]
 
 class ValidatedExtendedURIProxy(unittest.TestCase):
+    timeout = 240 # It takes longer than 120 seconds on Francois's arm box.
     K = 4
     M = 10
     SIZE = 200
@@ -270,6 +272,7 @@ class ValidatedExtendedURIProxy(unittest.TestCase):
         return defer.DeferredList(dl)
 
 class Encode(unittest.TestCase):
+    timeout = 2400 # It takes longer than 240 seconds on Zandr's ARM box.
 
     def do_encode(self, max_segment_size, datalen, NUM_SHARES, NUM_SEGMENTS,
                   expected_block_hashes, expected_share_hashes):
@@ -296,11 +299,13 @@ class Encode(unittest.TestCase):
                      (NUM_SEGMENTS-1)*segsize, len(data), NUM_SEGMENTS*segsize)
 
             shareholders = {}
+            servermap = {}
             for shnum in range(NUM_SHARES):
                 peer = FakeBucketReaderWriterProxy()
                 shareholders[shnum] = peer
+                servermap.setdefault(shnum, set()).add(peer.get_peerid())
                 all_shareholders.append(peer)
-            e.set_shareholders(shareholders)
+            e.set_shareholders(shareholders, servermap)
             return e.start()
         d.addCallback(_ready)
 
@@ -382,10 +387,9 @@ class Encode(unittest.TestCase):
         # 5 segments: 25, 25, 25, 25, 1
         return self.do_encode(25, 101, 100, 5, 15, 8)
 
-class PausingTarget(download.Data):
-    implements(IConsumer)
+class PausingConsumer(MemoryConsumer):
     def __init__(self):
-        download.Data.__init__(self)
+        MemoryConsumer.__init__(self)
         self.size = 0
         self.writes = 0
     def write(self, data):
@@ -396,33 +400,30 @@ class PausingTarget(download.Data):
             # last one (since then the _unpause timer will still be running)
             self.producer.pauseProducing()
             reactor.callLater(0.1, self._unpause)
-        return download.Data.write(self, data)
+        return MemoryConsumer.write(self, data)
     def _unpause(self):
         self.producer.resumeProducing()
-    def registerProducer(self, producer, streaming):
-        self.producer = producer
-    def unregisterProducer(self):
-        self.producer = None
 
-class PausingAndStoppingTarget(PausingTarget):
+class PausingAndStoppingConsumer(PausingConsumer):
     def write(self, data):
         self.producer.pauseProducing()
         reactor.callLater(0.5, self._stop)
     def _stop(self):
         self.producer.stopProducing()
 
-class StoppingTarget(PausingTarget):
+class StoppingConsumer(PausingConsumer):
     def write(self, data):
         self.producer.stopProducing()
 
 class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
+    timeout = 2400 # It takes longer than 240 seconds on Zandr's ARM box.
     def send_and_recover(self, k_and_happy_and_n=(25,75,100),
                          AVAILABLE_SHARES=None,
                          datalen=76,
                          max_segment_size=25,
                          bucket_modes={},
                          recover_mode="recover",
-                         target=None,
+                         consumer=None,
                          ):
         if AVAILABLE_SHARES is None:
             AVAILABLE_SHARES = k_and_happy_and_n[2]
@@ -431,7 +432,7 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
                       max_segment_size, bucket_modes, data)
         # that fires with (uri_extension_hash, e, shareholders)
         d.addCallback(self.recover, AVAILABLE_SHARES, recover_mode,
-                      target=target)
+                      consumer=consumer)
         # that fires with newdata
         def _downloaded((newdata, fd)):
             self.failUnless(newdata == data, str((len(newdata), len(data))))
@@ -459,12 +460,13 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
         def _ready(res):
             k,happy,n = e.get_param("share_counts")
             assert n == NUM_SHARES # else we'll be completely confused
-            all_peers = []
+            servermap = {}
             for shnum in range(NUM_SHARES):
                 mode = bucket_modes.get(shnum, "good")
-                peer = FakeBucketReaderWriterProxy(mode)
+                peer = FakeBucketReaderWriterProxy(mode, "peer%d" % shnum)
                 shareholders[shnum] = peer
-            e.set_shareholders(shareholders)
+                servermap.setdefault(shnum, set()).add(peer.get_peerid())
+            e.set_shareholders(shareholders, servermap)
             return e.start()
         d.addCallback(_ready)
         def _sent(res):
@@ -475,7 +477,7 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
         return d
 
     def recover(self, (res, key, shareholders), AVAILABLE_SHARES,
-                recover_mode, target=None):
+                recover_mode, consumer=None):
         verifycap = res
 
         if "corrupt_key" in recover_mode:
@@ -491,11 +493,12 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
                            total_shares=verifycap.total_shares,
                            size=verifycap.size)
 
-        client = FakeClient()
-        if not target:
-            target = download.Data()
-        target = download.DecryptingTarget(target, u.key)
-        fd = download.CiphertextDownloader(client, u.get_verify_cap(), target, monitor=Monitor())
+        sb = FakeStorageBroker()
+        if not consumer:
+            consumer = MemoryConsumer()
+        innertarget = download.ConsumerAdapter(consumer)
+        target = download.DecryptingTarget(innertarget, u.key)
+        fd = download.CiphertextDownloader(sb, u.get_verify_cap(), target, monitor=Monitor())
 
         # we manually cycle the CiphertextDownloader through a number of steps that
         # would normally be sequenced by a Deferred chain in
@@ -541,7 +544,8 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
 
         d.addCallback(fd._download_all_segments)
         d.addCallback(fd._done)
-        def _done(newdata):
+        def _done(t):
+            newdata = "".join(consumer.chunks)
             return (newdata, fd)
         d.addCallback(_done)
         return d
@@ -579,26 +583,26 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
         return self.send_and_recover(datalen=101)
 
     def test_pause(self):
-        # use a DownloadTarget that does pauseProducing/resumeProducing a few
-        # times, then finishes
-        t = PausingTarget()
-        d = self.send_and_recover(target=t)
+        # use a download target that does pauseProducing/resumeProducing a
+        # few times, then finishes
+        c = PausingConsumer()
+        d = self.send_and_recover(consumer=c)
         return d
 
     def test_pause_then_stop(self):
-        # use a DownloadTarget that pauses, then stops.
-        t = PausingAndStoppingTarget()
+        # use a download target that pauses, then stops.
+        c = PausingAndStoppingConsumer()
         d = self.shouldFail(download.DownloadStopped, "test_pause_then_stop",
                             "our Consumer called stopProducing()",
-                            self.send_and_recover, target=t)
+                            self.send_and_recover, consumer=c)
         return d
 
     def test_stop(self):
-        # use a DownloadTarget that does an immediate stop (ticket #473)
-        t = StoppingTarget()
+        # use a download targetthat does an immediate stop (ticket #473)
+        c = StoppingConsumer()
         d = self.shouldFail(download.DownloadStopped, "test_stop",
                             "our Consumer called stopProducing()",
-                            self.send_and_recover, target=t)
+                            self.send_and_recover, consumer=c)
         return d
 
     # the following tests all use 4-out-of-10 encoding
@@ -766,7 +770,7 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
     def test_lost_one_shareholder(self):
         # we have enough shareholders when we start, but one segment in we
         # lose one of them. The upload should still succeed, as long as we
-        # still have 'shares_of_happiness' peers left.
+        # still have 'servers_of_happiness' peers left.
         modemap = dict([(i, "good") for i in range(9)] +
                        [(i, "lost") for i in range(9, 10)])
         return self.send_and_recover((4,8,10), bucket_modes=modemap)
@@ -774,7 +778,7 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
     def test_lost_one_shareholder_early(self):
         # we have enough shareholders when we choose peers, but just before
         # we send the 'start' message, we lose one of them. The upload should
-        # still succeed, as long as we still have 'shares_of_happiness' peers
+        # still succeed, as long as we still have 'servers_of_happiness' peers
         # left.
         modemap = dict([(i, "good") for i in range(9)] +
                        [(i, "lost-early") for i in range(9, 10)])
@@ -788,7 +792,7 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
         d = self.send_and_recover((4,8,10), bucket_modes=modemap)
         def _done(res):
             self.failUnless(isinstance(res, Failure))
-            self.failUnless(res.check(NotEnoughSharesError), res)
+            self.failUnless(res.check(UploadUnhappinessError), res)
         d.addBoth(_done)
         return d
 
@@ -799,6 +803,6 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
         d = self.send_and_recover((4,8,10), bucket_modes=modemap)
         def _done(res):
             self.failUnless(isinstance(res, Failure))
-            self.failUnless(res.check(NotEnoughSharesError))
+            self.failUnless(res.check(UploadUnhappinessError))
         d.addBoth(_done)
         return d
